@@ -1,3 +1,4 @@
+import asyncio
 import binascii
 import codecs
 from collections import OrderedDict, defaultdict
@@ -25,9 +26,8 @@ from lndmanage.lib.ln_utilities import (
     extract_short_channel_id_from_string,
     convert_short_channel_id_to_channel_id,
     convert_channel_id_to_short_channel_id,
-    channel_unbalancedness_and_commit_fee
+    local_balance_to_unbalancedness
 )
-from lndmanage.lib.psbt import extract_psbt_inputs_outputs
 from lndmanage.lib.data_types import UTXO, AddressType
 from lndmanage.lib.user import yes_no_question
 from lndmanage.lib.utilities import convert_dictionary_number_strings_to_ints
@@ -44,28 +44,32 @@ NUM_MAX_FORWARDING_EVENTS = 100000
 OPEN_EXPIRY_TIME_MINUTES = 8
 
 
-class Node(object):
-    """Bare node object with attributes."""
-    def __init__(self):
-        logger.info("Initializing node interface.")
-        self.alias = ''
-        self.pub_key = ''
-        self.total_capacity = 0
-        self.total_local_balance = 0
-        self.total_remote_balance = 0
-        self.total_channels = 0
-        self.num_active_channels = 0
-        self.num_peers = 0
-        self.total_satoshis_received = 0
-        self.total_satoshis_sent = 0
-        self.total_private_channels = 0
-        self.total_active_channels = 0
-        self.blockheight = 0
-        self.channels = []
+class LndNode:
+    """Implements a synchronous/asynchronous interface to an lnd node."""
+    # rpcs
+    _routerrpc: lndrouterrpc.RouterStub
+    _rpc: lndrpc.LightningStub
+    _walletrpc: lndwalletkitrpc.WalletKitStub
+    _async_rpc: lndrpc.LightningStub
+    _async_routerrpc: lndrouterrpc.RouterStub
+    _async_channel: grpc.aio.Channel
+    _sync_channel: grpc.Channel
 
+    # attributes (TODO: clean up)
+    alias: str
+    pub_key: str
+    total_capacity: int = 0
+    total_local_balance: int = 0
+    total_remote_balance: int = 0
+    total_channels: int
+    num_active_channels: int
+    num_peers: int
+    total_satoshis_received: int = 0
+    total_satoshis_sent: int = 0
+    total_private_channels: int = 0
+    total_active_channels: int = 0
+    blockheight: int
 
-class LndNode(Node):
-    """Implements the node interface for LND."""
     def __init__(self, config_file: Optional[str] = None,
                  lnd_home: Optional[str] = None,
                  lnd_host: Optional[str] = None, regtest=False):
@@ -75,7 +79,6 @@ class LndNode(Node):
         :param lnd_host: lnd host of format "127.0.0.1:9735"
         :param regtest: if the node is representing a regtest node
         """
-        super().__init__()
         if config_file:
             self.config_file = config_file
             self.config = settings.read_config(self.config_file)
@@ -86,86 +89,105 @@ class LndNode(Node):
         self.lnd_host = lnd_host
         self.regtest = regtest
 
-        self._rpc = None
-        self._routerrpc = None
-        self.connect_rpcs()
-
-        self.network = Network(self)
-        self.update_blockheight()
-        self.set_info()
-
-    def connect_rpcs(self):
-        """
-        Establishes a connection to lnd using the hostname, tls certificate,
-        and admin macaroon defined in settings.
-        """
-        macaroons = True
-        os.environ['GRPC_SSL_CIPHER_SUITES'] = \
-            'ECDHE-RSA-AES128-GCM-SHA256:' \
-            'ECDHE-RSA-AES128-SHA256:' \
-            'ECDHE-RSA-AES256-SHA384:' \
-            'ECDHE-RSA-AES256-GCM-SHA384:' \
-            'ECDHE-ECDSA-AES128-GCM-SHA256:' \
-            'ECDHE-ECDSA-AES128-SHA256:' \
-            'ECDHE-ECDSA-AES256-SHA384:' \
-            'ECDHE-ECDSA-AES256-GCM-SHA384'
-
+        # configure lndmanage home: (TODO: separate into config)
         # if no lnd_home is given, then use the paths from the config,
         # else override them with default file paths in lnd_home
         if self.lnd_home is not None:
-            cert_file = os.path.join(self.lnd_home, 'tls.cert')
+            self.cert_file_path = os.path.join(self.lnd_home, 'tls.cert')
             bitcoin_network = 'regtest' if self.regtest else 'mainnet'
-            macaroon_file = os.path.join(
+            self.macaroon_file_path = os.path.join(
                 self.lnd_home, 'data/chain/bitcoin/',
                 bitcoin_network, 'admin.macaroon')
             if self.lnd_host is None:
-                raise ValueError(
-                    'if lnd_home is given, lnd_host must be given also')
-            lnd_host = self.lnd_host
+                raise ValueError('if lnd_home is given, lnd_host must be given')
         else:
-            cert_file = os.path.expanduser(self.config['network']['tls_cert_file'])
-            macaroon_file = \
-                os.path.expanduser(self.config['network']['admin_macaroon_file'])
-            lnd_host = self.config['network']['lnd_grpc_host']
+            self.cert_file_path = os.path.expanduser(
+                self.config['network']['tls_cert_file']
+            )
+            self.macaroon_file_path = os.path.expanduser(
+                self.config['network']['admin_macaroon_file']
+            )
+            self.lnd_host = self.config['network']['lnd_grpc_host']
 
+    def get_rpc_credentials(self) -> grpc.ChannelCredentials:
+        # read the tls certificate
         cert = None
         try:
-            with open(cert_file, 'rb') as f:
+            with open(self.cert_file_path, 'rb') as f:
                 cert = f.read()
         except FileNotFoundError:
             logger.error("tls.cert not found, please configure %s.",
                          self.config_file)
             exit(1)
 
-        if macaroons:
-            try:
-                with open(macaroon_file, 'rb') as f:
-                    macaroon_bytes = f.read()
-                    macaroon = codecs.encode(macaroon_bytes, 'hex')
-            except FileNotFoundError:
-                logger.error("admin.macaroon not found, please configure %s.",
-                             self.config_file)
-                exit(1)
+        # read the macaroon
+        try:
+            with open(self.macaroon_file_path, 'rb') as f:
+                macaroon_bytes = f.read()
+                macaroon = codecs.encode(macaroon_bytes, 'hex')
+        except FileNotFoundError:
+            logger.error("admin.macaroon not found, please configure %s.",
+                         self.config_file)
+            exit(1)
 
-            def metadata_callback(context, callback):
-                callback([('macaroon', macaroon)], None)
+        def metadata_callback(context, callback):
+            callback([('macaroon', macaroon)], None)
 
-            cert_creds = grpc.ssl_channel_credentials(cert)
-            auth_creds = grpc.metadata_call_credentials(metadata_callback)
-            creds = grpc.composite_channel_credentials(cert_creds, auth_creds)
+        cert_creds = grpc.ssl_channel_credentials(cert)
+        auth_creds = grpc.metadata_call_credentials(metadata_callback)
 
-        else:
-            creds = grpc.ssl_channel_credentials(cert)
+        return grpc.composite_channel_credentials(cert_creds, auth_creds)
 
-        # necessary to circumvent standard size limitation
-        channel = grpc.secure_channel(lnd_host, creds, options=[
-            ('grpc.max_receive_message_length', 50 * 1024 * 1024)
-        ])
+    async def connect_async_rpcs(self):
+        # This needs to be run within an async context, the loop is being used in the
+        # rpc connections.
+        logger.debug("Connecting async rpcs.")
+
+        self._async_channel = grpc.aio.secure_channel(
+            self.lnd_host, self.get_rpc_credentials(),
+            options=[('grpc.max_receive_message_length', 50 * 1024 * 1024)])
+
+        # establish async connections to rpc servers
+        self._async_rpc = lndrpc.LightningStub(self._async_channel)
+        self._async_routerrpc = lndrouterrpc.RouterStub(self._async_channel)
+
+    def connect_sync_rpcs(self):
+        self._sync_channel = grpc.secure_channel(
+            self.lnd_host, self.get_rpc_credentials(),
+            options=[('grpc.max_receive_message_length', 50 * 1024 * 1024)])
 
         # establish connections to rpc servers
-        self._rpc = lndrpc.LightningStub(channel)
-        self._routerrpc = lndrouterrpc.RouterStub(channel)
-        self._walletrpc = lndwalletkitrpc.WalletKitStub(channel)
+        self._rpc = lndrpc.LightningStub(self._sync_channel)
+        self._routerrpc = lndrouterrpc.RouterStub(self._sync_channel)
+        self._walletrpc = lndwalletkitrpc.WalletKitStub(self._sync_channel)
+
+    async def start(self):
+        logger.debug("Node interface starting.")
+
+        # connect rpcs
+        self.connect_sync_rpcs()
+        await self.connect_async_rpcs()
+
+        # init attributes that depend on rpc interaction
+        self.set_info()
+        self.network = Network(self)
+        self.update_blockheight()
+        self.set_channel_summary()
+
+    async def stop(self):
+        logger.debug("Disconnecting rpcs.")
+
+        self._sync_channel.close()
+        await self._async_channel.close()
+
+        # wait a bit to close all transports
+        await asyncio.sleep(0.01)
+
+    async def __aenter__(self):
+        await self.start()
+
+    async def __aexit__(self, exc_type, exc, tb):
+        await self.stop()
 
     def update_blockheight(self):
         info = self._rpc.GetInfo(lnd.GetInfoRequest())
@@ -245,19 +267,25 @@ class LndNode(Node):
             payment = self._routerrpc.SendToRouteV2(request, timeout=5 * 60)
         except _Rendezvous:
             raise PaymentTimeOut
+        except _InactiveRpcError:
+            raise PaymentTimeOut
         if payment.HasField('failure'):
             failure = payment.failure  # type: lnd.Failure.FailureCode
             logger.debug(f"Routing failure: {failure}")
             if failure.failure_source_index == 0:
                 raise OurNodeFailure("Not enough funds?")
-            if failure.code == 15:
-                raise exceptions.TemporaryChannelFailure(payment)
+            if failure.code == 12:
+                raise exceptions.FeeInsufficient(payment)
+            elif failure.code == 13:
+                raise exceptions.IncorrectCLTVExpiry(payment)
             elif failure.code == 14:
                 raise exceptions.ChannelDisabled(payment)
+            elif failure.code == 15:
+                raise exceptions.TemporaryChannelFailure(payment)
             elif failure.code == 18:
                 raise exceptions.UnknownNextPeer(payment)
-            elif failure.code == 12:
-                raise exceptions.FeeInsufficient(payment)
+            elif failure.code == 19:
+                raise exceptions.TemporaryNodeFailure(payment)
             else:
                 logger.info(failure)
                 raise Exception(f"Unknown error: code {failure.code}")
@@ -296,6 +324,7 @@ class LndNode(Node):
         self.num_active_channels = raw_info.num_active_channels
         self.num_peers = raw_info.num_peers
 
+    def set_channel_summary(self):
         # TODO: remove the following code and implement an advanced status
         all_channels = self.get_open_channels(
             active_only=False, public_only=False)
@@ -353,23 +382,28 @@ class LndNode(Node):
                 # age could be zero right after channel becomes pending
                 sent_received_per_week = 0
 
-            # determine policy
+            # determine policy:
             try:
                 edge_info = self.network.edges[c.chan_id]
                 # interested in node2
+                policies = edge_info['policies']
                 if edge_info['node1_pub'] == self.pub_key:
-                    policy_peer = edge_info['node2_policy']
-                    policy_local = edge_info['node1_policy']
+                    policy_peer = policies[edge_info['node2_pub'] > edge_info['node1_pub']]
+                    policy_local = policies[edge_info['node1_pub'] > edge_info['node2_pub']]
                 else:  # interested in node1
-                    policy_peer = edge_info['node1_policy']
-                    policy_local = edge_info['node2_policy']
+                    policy_peer = policies[edge_info['node1_pub'] > edge_info['node2_pub']]
+                    policy_local = policies[edge_info['node2_pub'] > edge_info['node1_pub']]
             except KeyError:
                 # if channel is unknown in describegraph
                 # we need to set the fees to some error value
-                policy_peer = {'fee_base_msat': float(-999),
-                               'fee_rate_milli_msat': float(999)}
-                policy_local = {'fee_base_msat': float(-999),
-                                'fee_rate_milli_msat': float(999)}
+                policy_peer = {
+                    'fee_base_msat': float(-999),
+                    'fee_rate_milli_msat': float(999)
+                }
+                policy_local = {
+                    'fee_base_msat': float(-999),
+                    'fee_rate_milli_msat': float(999)
+                }
 
             # calculate last update (days ago)
             def convert_to_days_ago(timestamp):
@@ -388,7 +422,7 @@ class LndNode(Node):
 
             # define unbalancedness |ub| large means very unbalanced
             channel_unbalancedness, our_commit_fee = \
-                channel_unbalancedness_and_commit_fee(
+                local_balance_to_unbalancedness(
                     c.local_balance, c.capacity, c.commit_fee, c.initiator)
             try:
                 uptime_lifetime_ratio = c.uptime / c.lifetime
@@ -410,6 +444,8 @@ class LndNode(Node):
                 'peer_fee_rate': policy_peer['fee_rate_milli_msat'],
                 'local_base_fee': policy_local['fee_base_msat'],
                 'local_fee_rate': policy_local['fee_rate_milli_msat'],
+                'local_chan_reserve_sat': c.local_chan_reserve_sat,
+                'remote_chan_reserve_sat': c.remote_chan_reserve_sat,
                 'initiator': c.initiator,
                 'last_update': last_update,
                 'last_update_local': last_update_local,
@@ -431,7 +467,7 @@ class LndNode(Node):
             sorted(channels.items(), key=lambda x: x[1]['alias']))
         return sorted_dict
 
-    def get_channel_id_to_node_id(self, open_only=False) -> Dict[int, str]:
+    def channel_id_to_node_id(self, open_only=False) -> Dict[int, str]:
         channel_id_to_node_id = {}
         closed_channels = self.get_closed_channels()
         open_channels = self.get_open_channels()
@@ -442,6 +478,12 @@ class LndNode(Node):
                 channel_id_to_node_id[cid] = c['remote_pubkey']
         return channel_id_to_node_id
 
+    def node_id_to_channel_ids(self, open_only=False) -> Dict[str, List[int]]:
+        node_channels_mapping = defaultdict(list)
+        for cid, nid in self.channel_id_to_node_id(open_only=open_only).items():
+            node_channels_mapping[nid].append(cid)
+        return node_channels_mapping
+
     def get_inactive_channels(self):
         """
         Returns all inactive channels.
@@ -450,7 +492,7 @@ class LndNode(Node):
         channels = self.get_open_channels(public_only=False, active_only=False)
         return {k: c for k, c in channels.items() if not c['active']}
 
-    def get_all_channels(self):
+    def get_all_channels(self, excluded_channels: List[int] = None):
         """
         Returns all active and inactive channels.
 
@@ -459,7 +501,7 @@ class LndNode(Node):
         channels = self.get_open_channels(public_only=False, active_only=False)
         return channels
 
-    def get_unbalanced_channels(self, unbalancedness_greater_than=0.0):
+    def get_unbalanced_channels(self, unbalancedness_greater_than=0.0, excluded_channels: List[int] = None, public_only=True, active_only=True):
         """
         Gets all channels which have an absolute unbalancedness
         (-1...1, -1 for outbound unbalanced, 1 for inbound unbalanced)
@@ -469,12 +511,13 @@ class LndNode(Node):
         :return: all channels which are more unbalanced than the specified interval
         """
         self.public_active_channels = \
-            self.get_open_channels(public_only=True, active_only=True)
-        unbalanced_channels = {
+            self.get_open_channels(public_only=public_only, active_only=active_only)
+        channels = {
             k: c for k, c in self.public_active_channels.items()
             if abs(c['unbalancedness']) >= unbalancedness_greater_than
         }
-        return unbalanced_channels
+        channels = {k: v for k, v in channels.items() if k not in (excluded_channels if excluded_channels else [])}
+        return channels
 
     def get_channel_fee_policies(self):
         """
@@ -739,201 +782,36 @@ class LndNode(Node):
 
     def open_channels(self, pubkeys: List[bytes],
                       amounts_sat: List[int],
-                      change_sat: int,
-                      utxos: Optional[List[UTXO]],
                       sat_per_vbyte: int,
-                      reckless=False,
-                      private=False):
-        """
-        Batch opens channels to other nodes.
-        Needs lnd compiled with walletrpc.
-        Should be used with lib.openchannels.ChannelOpener.open_channels
-        to sanitize method inputs.
+                      private=False, test=False):
+        channels = []
 
-        1. Construct PSBT for funding.
-        2. Fund and sanity check PSBT.
-        3. Verify PSBT by server.
-        4. Sign PSBT.
-        5. Tell server about signed PSBT.
-        6. Publish funding transaction.
-        7. Optionally abort funding and free locked utxos.
-        """
-        # at this stage we assume that we are already connected to the nodes
-
-        utxo_unlocking_needed = True
-        psbt_unfunded = None
-        utxo_leases = []
-        pending_chan_ids = []
-        addresses = []
-
-        logger.info(">>> Asking peers for channel opening.")
-        time_start = time.time()
-        try:
-            for n, (pk, amt) in enumerate(zip(pubkeys, amounts_sat)):
-                pending_chan_id = os.urandom(32)
-                pending_chan_ids.append(pending_chan_id)
-
-                # 1. Construct PSBT for funding.
-                if n == 0:  # for the first iteration we don't have a base psbt
-                    psbt_shim = lnd.PsbtShim(pending_chan_id=pending_chan_id, no_publish=True)
-                    shim = lnd.FundingShim(psbt_shim=psbt_shim)
-                else:
-                    psbt_shim = lnd.PsbtShim(pending_chan_id=pending_chan_id, base_psbt=psbt_unfunded, no_publish=True)
-                    shim = lnd.FundingShim(psbt_shim=psbt_shim)
-                request = lnd.OpenChannelRequest(
-                    node_pubkey=pk,
-                    local_funding_amount=amt,
-                    funding_shim=shim,
-                    private=private,
-                )
-                open_channel_stream = self._rpc.OpenChannel(request)
-
-                for chan_response in open_channel_stream:
-                    # first response should be that the funding psbt was created:
-                    if chan_response.HasField('psbt_fund'):
-                        logger.debug(f"   > peer {pk.hex()[:10]}...: got address: {chan_response.psbt_fund.funding_address}")
-                        addresses.append(chan_response.psbt_fund.funding_address)
-                        psbt_unfunded = chan_response.psbt_fund.psbt
-                        break
-
-            # 2. Fund and sanity check PSBT.
-            logger.info(f">>> Funding PSBT.")
-            inputs = [
-                lnd.OutPoint(
-                    txid_str=utxo.txid,
-                    output_index=utxo.output_index,
-                ) for utxo in utxos
-            ]
-            outputs = [
-                (addr, amount) for addr, amount in zip(addresses, amounts_sat)
-            ]
-            if change_sat:
-                change_address = self._rpc.NewAddress(lnd.NewAddressRequest(
-                    type=lnd.AddressType.WITNESS_PUBKEY_HASH)).address
-                outputs.append((change_address, change_sat))
-
-            fund_psbt = self._walletrpc.FundPsbt(lndwalletkit.FundPsbtRequest(
-                raw=lndwalletkit.TxTemplate(
-                    inputs=inputs,
-                    outputs=outputs,
-                ),
-                sat_per_vbyte=sat_per_vbyte,
+        logger.info(f">>> Opening channels at {sat_per_vbyte} sat per vbyte:")
+        for amount, pubkey in zip(amounts_sat, pubkeys):
+            logger.info(f"    {pubkey.hex()}: {amount} sat")
+            channels.append(lnd.BatchOpenChannel(
+                node_pubkey=pubkey,
+                local_funding_amount=amount,
+                push_sat=0,
+                private=private,
             ))
-            utxo_leases = fund_psbt.locked_utxos
 
-            # sanity checks
-            if fund_psbt.change_output_index != -1:
-                raise Exception("We shouldn't have an internal change output, please report this.")
+        logger.info("\n>>> WARNING: This feature is new, use at your own risk. "
+                    "Please check the above output carefully.\n")
+        logger.info("\n>>> Do you want to open the channel(s) (y/n)?")
+        if not test:
+            if not yes_no_question('no'):
+                return
 
-            num_inputs, num_outputs, psbt_amounts = extract_psbt_inputs_outputs(fund_psbt.funded_psbt)
-            logger.debug(f"    given inputs: {utxos}")
-            logger.debug(f"    inputs (psbt): {num_inputs}")
-            logger.debug(f"    outputs (psbt): {num_outputs}")
-            logger.debug(f"    amounts (psbt): {psbt_amounts}")
-            assert num_inputs == len(inputs)
-            assert num_outputs == len(outputs)
-            if change_sat:
-                if change_sat in psbt_amounts:
-                    psbt_amounts.remove(change_sat)
-                else:
-                    raise ValueError("Expected change, but couldn't find it.")
-            # order of outputs is not clear, comparing sets
-            assert set(amounts_sat) == set(psbt_amounts)
-            # add back change
-            psbt_amounts.append(change_sat)
-
-            fee = sum(utxo.amount_sat for utxo in utxos) - sum(psbt_amounts)
-            assert fee < 100000, f'Fee ({fee} sat) unreasonably high? Stopping.'
-
-            logger.info(f">>> WARNING: this is a relatively new feature, so "
-                        f"please check the generated PSBT by the following command:")
-            logger.info(f'bitcoin-cli decodepsbt "{str(binascii.b2a_base64(fund_psbt.funded_psbt).strip(), "utf-8")}"')
-            logger.info(f">>> You have {OPEN_EXPIRY_TIME_MINUTES} minutes from now to decide.\n")
-            logger.info("\n>>> Do you want to open the channel(s) (y/n)?")
-            if not reckless and not yes_no_question('no'):
-                raise InterruptedError("User canceled the process.")
-            time_end = time.time()
-            if time_end - time_start > OPEN_EXPIRY_TIME_MINUTES * 60:
-                raise InterruptedError("Time expired, aborted the channel opening process.")
-
-            # 3. Verify PSBT by server.
-            logger.info(f">>> Verifying PSBT.")
-            for p in pending_chan_ids:
-                response = str(self._rpc.FundingStateStep(
-                    lnd.FundingTransitionMsg(
-                        psbt_verify=lnd.FundingPsbtVerify(
-                            funded_psbt=fund_psbt.funded_psbt,
-                            pending_chan_id=p,
-                        ),
-                    )
-                )).strip()
-                if response:
-                    logger.debug(response)
-
-            # 4. Sign PSBT.
-            logger.info(f">>> Signing PSBT.")
-            finalize = self._walletrpc.FinalizePsbt(
-                lndwalletkit.FinalizePsbtRequest(funded_psbt=fund_psbt.funded_psbt))
-            raw_final_tx = finalize.raw_final_tx
-            psbt_signed = finalize.signed_psbt
-            logger.info(f"    Signed transaction:\n    {raw_final_tx.hex()}")
-            logger.info(f"    Signed psbt:\n    {str(binascii.b2a_base64(psbt_signed).strip(), 'utf-8')}")
-            logger.info(f"    Final transaction size: {len(raw_final_tx)} bytes")
-
-            # 5. Tell server about signed PSBT.
-            for p in pending_chan_ids:
-                response = str(self._rpc.FundingStateStep(
-                    lnd.FundingTransitionMsg(
-                        psbt_finalize=lnd.FundingPsbtFinalize(
-                            signed_psbt=psbt_signed,
-                            pending_chan_id=p,
-                        ),
-                    )
-                )).strip()
-                if response:
-                    logger.debug(f"   > Funding step response: {response}")
-
-            # 6. Publish funding transaction.
-            logger.info(f">>> Publishing transaction.")
-            self._walletrpc.PublishTransaction(lndwalletkit.Transaction(
-                tx_hex=raw_final_tx,
-                label='lndmanage: batch open'
-            ))
-            utxo_unlocking_needed = False
-
-        except grpc.RpcError as e:
-            logger.info(f"Error: {e}")
-
-        # 7. Optionally abort funding and free locked utxos.
-        finally:
-            if utxo_unlocking_needed:
-                logger.info(">>> Cleaning up.")
-                # cancel all funding reservations
-                for p in pending_chan_ids:
-                    try:
-                        self._rpc.FundingStateStep(
-                            lnd.FundingTransitionMsg(
-                                shim_cancel=lnd.FundingShimCancel(
-                                    pending_chan_id=p,
-                                ),
-                            )
-                        )
-                    except Exception as e:
-                        logger.info(e)
-                # unlock coins
-                for lease in utxo_leases:
-                    try:
-                        self._walletrpc.ReleaseOutput(
-                            lndwalletkit.ReleaseOutputRequest(
-                                id=lease.id,
-                                outpoint=lnd.OutPoint(
-                                    txid_str=lease.outpoint.txid_str,
-                                    output_index=lease.outpoint.output_index,
-                                ),
-                            )
-                        )
-                    except Exception as e:
-                        logger.info(e)
+        request = lnd.BatchOpenChannelRequest(
+            channels=channels,
+            sat_per_vbyte=sat_per_vbyte,
+            label='lndmanage: batch open',
+        )
+        response = self._rpc.BatchOpenChannel(request)
+        logger.info(f">>> Pending channels:")
+        for r in response.pending_channels:
+            logger.info(f"    {r.txid.hex()}:{r.output_index}")
 
     def _connect_nodes(self, pubkeys: List[str]) -> List[str]:
         """
