@@ -1,4 +1,5 @@
 #!/usr/bin/env python
+import asyncio
 import argparse
 import time
 import os
@@ -20,9 +21,10 @@ from lndmanage.lib.listings import ListChannels, ListPeers
 from lndmanage.lib.lncli import Lncli
 from lndmanage.lib.node import LndNode
 from lndmanage.lib.openchannels import ChannelOpener
-from lndmanage.lib.rebalance import Rebalancer
+from lndmanage.lib.rebalance import Rebalancer, DEFAULT_MAX_FEE_RATE, DEFAULT_AMOUNT_SAT
 from lndmanage.lib.recommend_nodes import RecommendNodes
 from lndmanage.lib.report import Report
+
 from lndmanage import settings
 
 import logging.config
@@ -117,7 +119,7 @@ class Parser(object):
             '--to-days-ago', default=0, type=int,
             help='time interval end (days ago)')
         parser_listchannels_forwardings.add_argument(
-            '--sort-by', default='f/w', type=str,
+            '--sort-by', default='fo/w', type=str,
             help='sort by column (look at description)')
 
         # subcmd: listchannels hygiene
@@ -140,7 +142,7 @@ class Parser(object):
             '--from-days-ago', default=60, type=int,
             help='time interval start (days ago)')
         self.parser_listpeers.add_argument(
-            '--sort-by', default='f/w', type=str,
+            '--sort-by', default='fio', type=str,
             help='sort by column (look at description)')
         listpeers_subparsers = self.parser_listpeers.add_subparsers(
             dest='subcmd')
@@ -159,18 +161,17 @@ class Parser(object):
         self.parser_rebalance = subparsers.add_parser(
             'rebalance', help='rebalance a channel',
             formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-        self.parser_rebalance.add_argument('channel', type=int,
-                                           help='channel_id')
+        self.parser_rebalance.add_argument('node_channel', type=str,
+                                           help='node id or channel id')
         self.parser_rebalance.add_argument(
-            '--max-fee-sat', type=int, default=20,
+            '--max-fee-sat', type=int, default=None,
             help='Sets the maximal fees in satoshis to be paid.')
         self.parser_rebalance.add_argument(
-            '--chunksize', type=float, default=1.0,
-            help='Specifies if the individual rebalance attempts should be '
-                 'split into smaller relative amounts. This increases success'
-                 ' rates, but also increases costs!')
+            '--amount', type=int, default=None,
+            help='Specifies the increase in local balance in sat. The amount can be'
+                 f'negative to decrease the local balance. Default: {DEFAULT_AMOUNT_SAT} sat.')
         self.parser_rebalance.add_argument(
-            '--max-fee-rate', type=range_limited_float_type, default=5E-5,
+            '--max-fee-rate', type=range_limited_float_type, default=DEFAULT_MAX_FEE_RATE,
             help='Sets the maximal effective fee rate to be paid.'
                  ' The effective fee rate is defined by '
                  '(base_fee + amt * fee_rate) / amt.')
@@ -178,45 +179,15 @@ class Parser(object):
             '--reckless', help='Execute action in the network.',
             action='store_true')
         self.parser_rebalance.add_argument(
-            '--allow-unbalancing',
-            help=f'Allow channels to get an unbalancedness'
-            f' up to +-{settings.UNBALANCED_CHANNEL}.',
+            '--force',
+            help=f"Allow rebalances that are uneconomic.",
             action='store_true')
         self.parser_rebalance.add_argument(
-            '--target', help=f'This feature is still experimental! '
-            f'The unbalancedness target is between [-1, 1]. '
+            '--target', help=f'The unbalancedness target is between [-1, 1]. '
             f'A target of -1 leads to a maximal local balance, a target of 0 '
             f'to a 50:50 balanced channel and a target of 1 to a maximal '
             f'remote balance. Default is a target of 0.',
-            type=unbalanced_float, default=0.0)
-        rebalancing_strategies = ['most-affordable-first',
-                                  'lowest-feerate-first', 'match-unbalanced']
-        self.parser_rebalance.add_argument(
-            '--strategy',
-            help=f'Rebalancing strategy.',
-            choices=rebalancing_strategies, type=str, default=None)
-
-        # cmd: circle
-        self.parser_circle = subparsers.add_parser(
-            'circle', help='circular self-payment',
-            formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-        self.parser_circle.add_argument('channel_from', type=int,
-                                        help='channel_from')
-        self.parser_circle.add_argument('channel_to', type=int,
-                                        help='channel_from')
-        self.parser_circle.add_argument('amt_sat', type=int,
-                                        help='amount in satoshis')
-        self.parser_circle.add_argument(
-            '--max-fee-sat', type=int, default=20,
-            help='Sets the maximal fees in satoshis to be paid.')
-        self.parser_circle.add_argument(
-            '--max-fee-rate', type=range_limited_float_type, default=5E-5,
-            help='Sets the maximal effective fee rate to be paid. '
-                 'The effective fee rate is defined by '
-                 '(base_fee + amt * fee_rate) / amt.')
-        self.parser_circle.add_argument(
-            '--reckless', help='Execute action in the network.',
-            action='store_true')
+            type=unbalanced_float, default=None)
 
         # cmd: recommend-nodes
         self.parser_recommend_nodes = subparsers.add_parser(
@@ -357,39 +328,23 @@ class Parser(object):
         # cmd: openchannels
         self.parser_openchannels = subparsers.add_parser(
             'openchannels',
-            help='opens multiple channels with UTXO control',
+            help='opens multiple channels',
             formatter_class=argparse.ArgumentDefaultsHelpFormatter)
-        self.parser_openchannels.add_argument(
-            '--utxos',
-            type=str,
-            help='Comma-separated list of utxos of format '
-                 'txid:output_index,txid:output_index which belong to the LND '
-                 'wallet (see lncli listunspent).')
         self.parser_openchannels.add_argument(
             '--amounts',
             type=str,
-            help='Comma-separated list of channel amounts in sat 1000000,2000000,... '
-                 'A list of amounts with sum < 100 denotes relative amounts that are rescaled '
-                 'to the sum of all UTXOs provided, such that no change is created. '
-                 'If the amounts provided are larger than the funds available, '
-                 'they will automatically be reduced proportionally such that no change is created. '
-                 'Change is created if anchor reserve funds would be depleted.')
+            help='Comma-separated list of channel amounts in sat 1000000,2000000,... ',
+        )
         self.parser_openchannels.add_argument(
             '--total-amount',
             type=int,
             help='Total amount in sat to open channels.'
-                 'Amounts and total amount flags are mutually exclusive. '
-                 'If none of the two amounts are given, all available UTXOs '
-                 'will be spent.')
+                 'Amounts and total amount flags are mutually exclusive.')
         self.parser_openchannels.add_argument(
             '--sat-per-vbyte',
             type=int,
             default=1,
             help='The fee rate in sat per vbyte that will be targeted.')
-        self.parser_openchannels.add_argument(
-            '--reckless',
-            help='If this flag is provided, the user will not be asked '
-                 'to agree to channel opening.', action='store_true')
         self.parser_openchannels.add_argument(
             '--private', help='The channels will not be announced to the network.',
             action='store_true')
@@ -467,8 +422,7 @@ class Parser(object):
 
         # look in lndmanage home folder after lncli
         if os.access(lncli_candidate, os.X_OK):
-            if self.lncli_path:
-                self.lncli_path = lncli_candidate
+            self.lncli_path = lncli_candidate
         # look in PATH
         else:
             path = spawn.find_executable('lncli')
@@ -477,7 +431,7 @@ class Parser(object):
     def parse_arguments(self):
         return self.parser.parse_args()
 
-    def run_commands(self, node, args):
+    async def run_commands(self, node, args):
         # program execution
         if args.loglevel:
             # update the loglevel of the stdout handler to the user choice
@@ -543,15 +497,16 @@ class Parser(object):
             if args.target:
                 logger.warning("Warning: Target is set, this is still an "
                                "experimental feature.")
-            rebalancer = Rebalancer(node, args.max_fee_rate, args.max_fee_sat)
+            rebalancer = Rebalancer(node, args.max_fee_rate, args.max_fee_sat, args.force)
             try:
                 rebalancer.rebalance(
-                    args.channel,
+                    args.node_channel,
                     dry=not args.reckless,
-                    chunksize=args.chunksize,
                     target=args.target,
-                    allow_unbalancing=args.allow_unbalancing,
-                    strategy=args.strategy)
+                    amount_sat=args.amount
+                )
+            except ValueError as e:
+                logger.error(e)
             except TooExpensive as e:
                 logger.error(f"Too expensive: {e}")
             except RebalanceFailure as e:
@@ -559,29 +514,6 @@ class Parser(object):
             except KeyboardInterrupt:
                 pass
 
-        elif args.cmd == 'circle':
-            rebalancer = Rebalancer(node, args.max_fee_rate, args.max_fee_sat)
-            invoice = node.get_rebalance_invoice(memo='circular payment')
-            payment_hash, payment_address = invoice.r_hash, invoice.payment_addr
-            try:
-                rebalancer.rebalance_two_channels(
-                    args.channel_from, args.channel_to,
-                    args.amt_sat, payment_hash, payment_address,
-                    args.max_fee_sat,
-                    dry=not args.reckless)
-            except DryRun:
-                logger.info("This was just a dry run.")
-            except TooExpensive:
-                logger.error(
-                    "Too expensive: consider to raise --max-fee-sat or "
-                    "--max-fee-rate.")
-            except RebalancingTrialsExhausted:
-                logger.error(
-                    f"Rebalancing trials exhausted (number of trials: "
-                    f"{settings.REBALANCING_TRIALS}).")
-            except PaymentTimeOut:
-                logger.error("Payment failed because the payment timed out.")
-
         elif args.cmd == 'recommend-nodes':
             if not args.subcmd:
                 self.parser_recommend_nodes.print_help()
@@ -628,10 +560,8 @@ class Parser(object):
                 channel_opener.open_channels(
                     pubkeys=args.pubkeys,
                     amounts=args.amounts,
-                    utxos=args.utxos,
                     sat_per_vbyte=args.sat_per_vbyte,
                     total_amount=args.total_amount,
-                    reckless=args.reckless,
                     private=args.private,
                 )
             except Exception as e:
@@ -657,7 +587,7 @@ class Parser(object):
             )
 
 
-def main():
+async def _main():
     parser = Parser()
 
     # config.ini is expected to be in home/.lndmanage directory
@@ -667,8 +597,10 @@ def main():
     if len(sys.argv) > 1:
         # take arguments from sys.argv
         args = parser.parse_arguments()
-        node = LndNode(config_file=config_file)
-        parser.run_commands(node, args)
+
+        lndnode = LndNode(config_file=config_file)
+        async with lndnode:
+            await parser.run_commands(lndnode, args)
 
     # otherwise enter an interactive mode
     else:
@@ -681,50 +613,55 @@ def main():
 
         logger.info("Running in interactive mode. "
                     "You can type 'help' or 'exit'.")
-        node = LndNode(config_file=config_file)
 
-        if parser.lncli_path:
-            logger.info("Enabled lncli: using " + parser.lncli_path)
+        lndnode = LndNode(config_file=config_file)
+        async with lndnode:
+            if parser.lncli_path:
+                logger.info("Enabled lncli: using " + parser.lncli_path)
 
-        while True:
-            try:
-                user_input = input("$ lndmanage ")
-            except KeyboardInterrupt:
-                logger.info("")
-                continue
-            except EOFError:
-                readline.write_history_file(history_file)
-                logger.info("exit")
-                return 0
-
-            if not user_input or user_input in ['help', '-h', '--help']:
-                parser.parser.print_help()
-                continue
-            elif user_input == 'exit':
-                readline.write_history_file(history_file)
-                return 0
-
-            args_list = user_input.split(" ")
-
-            # lncli execution
-            if args_list[0] == 'lncli':
-                if parser.lncli_path:
-                    lncli = Lncli(parser.lncli_path, config_file)
-                    lncli.lncli(args_list[1:])
+            while True:
+                try:
+                    user_input = input("$ lndmanage ")
+                except KeyboardInterrupt:
+                    logger.info("")
                     continue
-                else:
-                    logger.info("lncli not enabled, put lncli in PATH or in ~/.lndmanage")
+                except EOFError:
+                    readline.write_history_file(history_file)
+                    logger.info("exit")
+                    return 0
+
+                if not user_input or user_input in ['help', '-h', '--help']:
+                    parser.parser.print_help()
                     continue
-            try:
-                # need to run with parse_known_args to get an exception
-                args = parser.parser.parse_args(args_list)
-                parser.run_commands(node, args)
-            except SystemExit:
-                # argparse may raise SystemExit on incorrect user input,
-                # which is a graceful exit. The user gets the standard output
-                # from argparse of what went wrong.
-                continue
+                elif user_input == 'exit':
+                    readline.write_history_file(history_file)
+                    return 0
+
+                args_list = user_input.split(" ")
+
+                # lncli execution
+                if args_list[0] == 'lncli':
+                    if parser.lncli_path:
+                        lncli = Lncli(parser.lncli_path, config_file)
+                        lncli.lncli(args_list[1:])
+                        continue
+                    else:
+                        logger.info("lncli not enabled, put lncli in PATH or in ~/.lndmanage")
+                        continue
+                try:
+                    # need to run with parse_known_args to get an exception
+                    args = parser.parser.parse_args(args_list)
+                    await parser.run_commands(lndnode, args)
+                except SystemExit:
+                    # argparse may raise SystemExit on incorrect user input,
+                    # which is a graceful exit. The user gets the standard output
+                    # from argparse of what went wrong.
+                    continue
+
+
+def main():
+    asyncio.run(_main())
 
 
 if __name__ == '__main__':
-    main()
+    asyncio.run(_main())
